@@ -1,6 +1,6 @@
 /* libFLAC - Free Lossless Audio Codec library
  * Copyright (C) 2000-2009  Josh Coalson
- * Copyright (C) 2011-2018  Xiph.Org Foundation
+ * Copyright (C) 2011-2025  Xiph.Org Foundation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -39,6 +39,7 @@
 #include "private/bitmath.h"
 #include "private/bitreader.h"
 #include "private/crc.h"
+#include "private/cpu.h"
 #include "private/macros.h"
 #include "FLAC/assert.h"
 #include "share/compat.h"
@@ -111,6 +112,9 @@ struct FLAC__BitReader {
 	uint32_t read_crc16; /* the running frame CRC */
 	uint32_t crc16_offset; /* the number of words in the current buffer that should not be CRC'd */
 	uint32_t crc16_align; /* the number of bits in the current consumed word that should not be CRC'd */
+	FLAC__bool read_limit_set; /* whether reads are limited */
+	uint32_t read_limit; /* the remaining size of what can be read */
+	uint32_t last_seen_framesync; /* the location of the last seen framesync, if it is in the buffer, in bits from front of buffer */
 	FLAC__BitReaderReadCallback read_callback;
 	void *client_data;
 };
@@ -155,9 +159,16 @@ static FLAC__bool bitreader_read_from_client_(FLAC__BitReader *br)
 	uint32_t start, end;
 	size_t bytes;
 	FLAC__byte *target;
+#if WORDS_BIGENDIAN
+#else
+	brword preswap_backup;
+#endif
 
 	/* first shift the unconsumed buffer data toward the front as much as possible */
 	if(br->consumed_words > 0) {
+		/* invalidate last seen framesync */
+		br->last_seen_framesync = -1;
+
 		crc16_update_block_(br); /* CRC consumed words */
 
 		start = br->consumed_words;
@@ -186,6 +197,7 @@ static FLAC__bool bitreader_read_from_client_(FLAC__BitReader *br)
 	 */
 #if WORDS_BIGENDIAN
 #else
+	preswap_backup = br->buffer[br->words];
 	if(br->bytes)
 		br->buffer[br->words] = SWAP_BE_WORD_TO_HOST(br->buffer[br->words]);
 #endif
@@ -198,8 +210,16 @@ static FLAC__bool bitreader_read_from_client_(FLAC__BitReader *br)
 	 */
 
 	/* read in the data; note that the callback may return a smaller number of bytes */
-	if(!br->read_callback(target, &bytes, br->client_data))
+	if(!br->read_callback(target, &bytes, br->client_data)){
+		/* Despite the read callback failing, the data in the target
+		 * might be used later, when the buffer is rewound. Therefore
+		 * we revert the swap that was just done */
+#if WORDS_BIGENDIAN
+#else
+		br->buffer[br->words] = preswap_backup;
+#endif
 		return false;
+	}
 
 	/* after reading bytes 66 77 88 99 AA BB CC DD EE FF from the client:
 	 *   bitstream :  11 22 33 44 55 66 77 88 99 AA BB CC DD EE FF
@@ -275,6 +295,9 @@ FLAC__bool FLAC__bitreader_init(FLAC__BitReader *br, FLAC__BitReaderReadCallback
 		return false;
 	br->read_callback = rcb;
 	br->client_data = cd;
+	br->read_limit_set = false;
+	br->read_limit = -1;
+	br->last_seen_framesync = -1;
 
 	return true;
 }
@@ -291,42 +314,36 @@ void FLAC__bitreader_free(FLAC__BitReader *br)
 	br->consumed_words = br->consumed_bits = 0;
 	br->read_callback = 0;
 	br->client_data = 0;
+	br->read_limit_set = false;
+	br->read_limit = -1;
+	br->last_seen_framesync = -1;
 }
 
 FLAC__bool FLAC__bitreader_clear(FLAC__BitReader *br)
 {
 	br->words = br->bytes = 0;
 	br->consumed_words = br->consumed_bits = 0;
+	br->read_limit_set = false;
+	br->read_limit = -1;
+	br->last_seen_framesync = -1;
 	return true;
 }
 
-void FLAC__bitreader_dump(const FLAC__BitReader *br, FILE *out)
+void FLAC__bitreader_set_framesync_location(FLAC__BitReader *br)
 {
-	uint32_t i, j;
-	if(br == 0) {
-		fprintf(out, "bitreader is NULL\n");
+	br->last_seen_framesync = br->consumed_words * FLAC__BYTES_PER_WORD + br->consumed_bits / 8;
+}
+
+FLAC__bool FLAC__bitreader_rewind_to_after_last_seen_framesync(FLAC__BitReader *br)
+{
+	if(br->last_seen_framesync == (uint32_t)-1) {
+		br->consumed_words = br->consumed_bits = 0;
+		return false;
 	}
 	else {
-		fprintf(out, "bitreader: capacity=%u words=%u bytes=%u consumed: words=%u, bits=%u\n", br->capacity, br->words, br->bytes, br->consumed_words, br->consumed_bits);
-
-		for(i = 0; i < br->words; i++) {
-			fprintf(out, "%08X: ", i);
-			for(j = 0; j < FLAC__BITS_PER_WORD; j++)
-				if(i < br->consumed_words || (i == br->consumed_words && j < br->consumed_bits))
-					fprintf(out, ".");
-				else
-					fprintf(out, "%01d", br->buffer[i] & ((brword)1 << (FLAC__BITS_PER_WORD-j-1)) ? 1:0);
-			fprintf(out, "\n");
-		}
-		if(br->bytes > 0) {
-			fprintf(out, "%08X: ", i);
-			for(j = 0; j < br->bytes*8; j++)
-				if(i < br->consumed_words || (i == br->consumed_words && j < br->consumed_bits))
-					fprintf(out, ".");
-				else
-					fprintf(out, "%01d", br->buffer[i] & ((brword)1 << (br->bytes*8-j-1)) ? 1:0);
-			fprintf(out, "\n");
-		}
+		br->consumed_words = (br->last_seen_framesync + 1) / FLAC__BYTES_PER_WORD;
+		br->consumed_bits  = ((br->last_seen_framesync + 1) % FLAC__BYTES_PER_WORD) * 8;
+		return true;
 	}
 }
 
@@ -376,6 +393,28 @@ inline uint32_t FLAC__bitreader_get_input_bits_unconsumed(const FLAC__BitReader
 	return (br->words-br->consumed_words)*FLAC__BITS_PER_WORD + br->bytes*8 - br->consumed_bits;
 }
 
+void FLAC__bitreader_set_limit(FLAC__BitReader *br, uint32_t limit)
+{
+	br->read_limit = limit;
+	br->read_limit_set = true;
+}
+
+void FLAC__bitreader_remove_limit(FLAC__BitReader *br)
+{
+	br->read_limit_set = false;
+	br->read_limit = -1;
+}
+
+uint32_t FLAC__bitreader_limit_remaining(FLAC__BitReader *br)
+{
+	FLAC__ASSERT(br->read_limit_set);
+	return br->read_limit;
+}
+void FLAC__bitreader_limit_invalidate(FLAC__BitReader *br)
+{
+	br->read_limit = -1;
+}
+
 FLAC__bool FLAC__bitreader_read_raw_uint32(FLAC__BitReader *br, FLAC__uint32 *val, uint32_t bits)
 {
 	FLAC__ASSERT(0 != br);
@@ -393,6 +432,15 @@ FLAC__bool FLAC__bitreader_read_raw_uint32(FLAC__BitReader *br, FLAC__uint32 *va
 		return true;
 	}
 
+	if(br->read_limit_set && br->read_limit < (uint32_t)-1){
+		if(br->read_limit < bits) {
+			br->read_limit = -1;
+			return false;
+		}
+		else
+			br->read_limit -= bits;
+	}
+
 	while((br->words-br->consumed_words)*FLAC__BITS_PER_WORD + br->bytes*8 - br->consumed_bits < bits) {
 		if(!bitreader_read_from_client_(br))
 			return false;
@@ -465,7 +513,7 @@ FLAC__bool FLAC__bitreader_read_raw_int32(FLAC__BitReader *br, FLAC__int32 *val,
 		return false;
 	/* sign-extend *val assuming it is currently bits wide. */
 	/* From: https://graphics.stanford.edu/~seander/bithacks.html#FixedSignExtend */
-	mask = bits >= 33 ? 0 : 1u << (bits - 1);
+	mask = bits >= 33 ? 0 : 1lu << (bits - 1);
 	*val = (uval ^ mask) - mask;
 	return true;
 }
@@ -491,6 +539,19 @@ FLAC__bool FLAC__bitreader_read_raw_uint64(FLAC__BitReader *br, FLAC__uint64 *va
 	return true;
 }
 
+FLAC__bool FLAC__bitreader_read_raw_int64(FLAC__BitReader *br, FLAC__int64 *val, uint32_t bits)
+{
+	FLAC__uint64 uval, mask;
+	/* OPT: inline raw uint64 code here, or make into a macro if possible in the .h file */
+	if (bits < 1 || ! FLAC__bitreader_read_raw_uint64(br, &uval, bits))
+		return false;
+	/* sign-extend *val assuming it is currently bits wide. */
+	/* From: https://graphics.stanford.edu/~seander/bithacks.html#FixedSignExtend */
+	mask = bits >= 65 ? 0 : 1llu << (bits - 1);
+	*val = (uval ^ mask) - mask;
+	return true;
+}
+
 inline FLAC__bool FLAC__bitreader_read_uint32_little_endian(FLAC__BitReader *br, FLAC__uint32 *val)
 {
 	FLAC__uint32 x8, x32 = 0;
@@ -559,6 +620,13 @@ FLAC__bool FLAC__bitreader_skip_byte_block_aligned_no_crc(FLAC__BitReader *br, u
 	FLAC__ASSERT(0 != br->buffer);
 	FLAC__ASSERT(FLAC__bitreader_is_consumed_byte_aligned(br));
 
+	if(br->read_limit_set && br->read_limit < (uint32_t)-1){
+		if(br->read_limit < nvals*8){
+			br->read_limit = -1;
+			return false;
+		}
+	}
+
 	/* step 1: skip over partial head word to get word aligned */
 	while(nvals && br->consumed_bits) { /* i.e. run until we read 'nvals' bytes or we hit the end of the head word */
 		if(!FLAC__bitreader_read_raw_uint32(br, &x, 8))
@@ -567,11 +635,14 @@ FLAC__bool FLAC__bitreader_skip_byte_block_aligned_no_crc(FLAC__BitReader *br, u
 	}
 	if(0 == nvals)
 		return true;
+
 	/* step 2: skip whole words in chunks */
 	while(nvals >= FLAC__BYTES_PER_WORD) {
 		if(br->consumed_words < br->words) {
 			br->consumed_words++;
 			nvals -= FLAC__BYTES_PER_WORD;
+			if(br->read_limit_set)
+				br->read_limit -= FLAC__BITS_PER_WORD;
 		}
 		else if(!bitreader_read_from_client_(br))
 			return false;
@@ -594,6 +665,13 @@ FLAC__bool FLAC__bitreader_read_byte_block_aligned_no_crc(FLAC__BitReader *br, F
 	FLAC__ASSERT(0 != br->buffer);
 	FLAC__ASSERT(FLAC__bitreader_is_consumed_byte_aligned(br));
 
+	if(br->read_limit_set && br->read_limit < (uint32_t)-1){
+		if(br->read_limit < nvals*8){
+			br->read_limit = -1;
+			return false;
+		}
+	}
+
 	/* step 1: read from partial head word to get word aligned */
 	while(nvals && br->consumed_bits) { /* i.e. run until we read 'nvals' bytes or we hit the end of the head word */
 		if(!FLAC__bitreader_read_raw_uint32(br, &x, 8))
@@ -627,6 +705,8 @@ FLAC__bool FLAC__bitreader_read_byte_block_aligned_no_crc(FLAC__BitReader *br, F
 #endif
 			val += FLAC__BYTES_PER_WORD;
 			nvals -= FLAC__BYTES_PER_WORD;
+			if(br->read_limit_set)
+				br->read_limit -= FLAC__BITS_PER_WORD;
 		}
 		else if(!bitreader_read_from_client_(br))
 			return false;
@@ -721,6 +801,7 @@ FLAC__bool FLAC__bitreader_read_unary_unsigned(FLAC__BitReader *br, uint32_t *va
 }
 #endif
 
+#if 0 /* unused */
 FLAC__bool FLAC__bitreader_read_rice_signed(FLAC__BitReader *br, int *val, uint32_t parameter)
 {
 	FLAC__uint32 lsbs = 0, msbs = 0;
@@ -747,146 +828,17 @@ FLAC__bool FLAC__bitreader_read_rice_signed(FLAC__BitReader *br, int *val, uint3
 
 	return true;
 }
+#endif
 
 /* this is by far the most heavily used reader call.  it ain't pretty but it's fast */
 FLAC__bool FLAC__bitreader_read_rice_signed_block(FLAC__BitReader *br, int vals[], uint32_t nvals, uint32_t parameter)
-{
-	/* try and get br->consumed_words and br->consumed_bits into register;
-	 * must remember to flush them back to *br before calling other
-	 * bitreader functions that use them, and before returning */
-	uint32_t cwords, words, lsbs, msbs, x, y;
-	uint32_t ucbits; /* keep track of the number of unconsumed bits in word */
-	brword b;
-	int *val, *end;
+#include "deduplication/bitreader_read_rice_signed_block.c"
 
-	FLAC__ASSERT(0 != br);
-	FLAC__ASSERT(0 != br->buffer);
-	/* WATCHOUT: code does not work with <32bit words; we can make things much faster with this assertion */
-	FLAC__ASSERT(FLAC__BITS_PER_WORD >= 32);
-	FLAC__ASSERT(parameter < 32);
-	/* the above two asserts also guarantee that the binary part never straddles more than 2 words, so we don't have to loop to read it */
-
-	val = vals;
-	end = vals + nvals;
-
-	if(parameter == 0) {
-		while(val < end) {
-			/* read the unary MSBs and end bit */
-			if(!FLAC__bitreader_read_unary_unsigned(br, &msbs))
-				return false;
-
-			*val++ = (int)(msbs >> 1) ^ -(int)(msbs & 1);
-		}
-
-		return true;
-	}
-
-	FLAC__ASSERT(parameter > 0);
-
-	cwords = br->consumed_words;
-	words = br->words;
-
-	/* if we've not consumed up to a partial tail word... */
-	if(cwords >= words) {
-		x = 0;
-		goto process_tail;
-	}
-
-	ucbits = FLAC__BITS_PER_WORD - br->consumed_bits;
-	b = br->buffer[cwords] << br->consumed_bits;  /* keep unconsumed bits aligned to left */
-
-	while(val < end) {
-		/* read the unary MSBs and end bit */
-		x = y = COUNT_ZERO_MSBS2(b);
-		if(x == FLAC__BITS_PER_WORD) {
-			x = ucbits;
-			do {
-				/* didn't find stop bit yet, have to keep going... */
-				cwords++;
-				if (cwords >= words)
-					goto incomplete_msbs;
-				b = br->buffer[cwords];
-				y = COUNT_ZERO_MSBS2(b);
-				x += y;
-			} while(y == FLAC__BITS_PER_WORD);
-		}
-		b <<= y;
-		b <<= 1; /* account for stop bit */
-		ucbits = (ucbits - x - 1) % FLAC__BITS_PER_WORD;
-		msbs = x;
-
-		/* read the binary LSBs */
-		x = (FLAC__uint32)(b >> (FLAC__BITS_PER_WORD - parameter)); /* parameter < 32, so we can cast to 32-bit uint32_t */
-		if(parameter <= ucbits) {
-			ucbits -= parameter;
-			b <<= parameter;
-		} else {
-			/* there are still bits left to read, they will all be in the next word */
-			cwords++;
-			if (cwords >= words)
-				goto incomplete_lsbs;
-			b = br->buffer[cwords];
-			ucbits += FLAC__BITS_PER_WORD - parameter;
-			x |= (FLAC__uint32)(b >> ucbits);
-			b <<= FLAC__BITS_PER_WORD - ucbits;
-		}
-		lsbs = x;
-
-		/* compose the value */
-		x = (msbs << parameter) | lsbs;
-		*val++ = (int)(x >> 1) ^ -(int)(x & 1);
-
-		continue;
-
-		/* at this point we've eaten up all the whole words */
-process_tail:
-		do {
-			if(0) {
-incomplete_msbs:
-				br->consumed_bits = 0;
-				br->consumed_words = cwords;
-			}
-
-			/* read the unary MSBs and end bit */
-			if(!FLAC__bitreader_read_unary_unsigned(br, &msbs))
-				return false;
-			msbs += x;
-			x = ucbits = 0;
-
-			if(0) {
-incomplete_lsbs:
-				br->consumed_bits = 0;
-				br->consumed_words = cwords;
-			}
-
-			/* read the binary LSBs */
-			if(!FLAC__bitreader_read_raw_uint32(br, &lsbs, parameter - ucbits))
-				return false;
-			lsbs = x | lsbs;
-
-			/* compose the value */
-			x = (msbs << parameter) | lsbs;
-			*val++ = (int)(x >> 1) ^ -(int)(x & 1);
-			x = 0;
-
-			cwords = br->consumed_words;
-			words = br->words;
-			ucbits = FLAC__BITS_PER_WORD - br->consumed_bits;
-			b = cwords < br->capacity ? br->buffer[cwords] << br->consumed_bits : 0;
-		} while(cwords >= words && val < end);
-	}
-
-	if(ucbits == 0 && cwords < words) {
-		/* don't leave the head word with no unconsumed bits */
-		cwords++;
-		ucbits = FLAC__BITS_PER_WORD;
-	}
-
-	br->consumed_bits = FLAC__BITS_PER_WORD - ucbits;
-	br->consumed_words = cwords;
-
-	return true;
-}
+#ifdef FLAC__BMI2_SUPPORTED
+FLAC__SSE_TARGET("bmi2")
+FLAC__bool FLAC__bitreader_read_rice_signed_block_bmi2(FLAC__BitReader *br, int vals[], uint32_t nvals, uint32_t parameter)
+#include "deduplication/bitreader_read_rice_signed_block.c"
+#endif
 
 #if 0 /* UNUSED */
 FLAC__bool FLAC__bitreader_read_golomb_signed(FLAC__BitReader *br, int *val, uint32_t parameter)
@@ -987,23 +939,23 @@ FLAC__bool FLAC__bitreader_read_utf8_uint32(FLAC__BitReader *br, FLAC__uint32 *v
 		v = x;
 		i = 0;
 	}
-	else if(x & 0xC0 && !(x & 0x20)) { /* 110xxxxx */
+	else if((x & 0xE0) == 0xC0) { /* 110xxxxx */
 		v = x & 0x1F;
 		i = 1;
 	}
-	else if(x & 0xE0 && !(x & 0x10)) { /* 1110xxxx */
+	else if((x & 0xF0) == 0xE0) { /* 1110xxxx */
 		v = x & 0x0F;
 		i = 2;
 	}
-	else if(x & 0xF0 && !(x & 0x08)) { /* 11110xxx */
+	else if((x & 0xF8) == 0xF0) { /* 11110xxx */
 		v = x & 0x07;
 		i = 3;
 	}
-	else if(x & 0xF8 && !(x & 0x04)) { /* 111110xx */
+	else if((x & 0xFC) == 0xF8) { /* 111110xx */
 		v = x & 0x03;
 		i = 4;
 	}
-	else if(x & 0xFC && !(x & 0x02)) { /* 1111110x */
+	else if((x & 0xFE) == 0xFC) { /* 1111110x */
 		v = x & 0x01;
 		i = 5;
 	}
@@ -1042,27 +994,27 @@ FLAC__bool FLAC__bitreader_read_utf8_uint64(FLAC__BitReader *br, FLAC__uint64 *v
 		v = x;
 		i = 0;
 	}
-	else if(x & 0xC0 && !(x & 0x20)) { /* 110xxxxx */
+	else if((x & 0xE0) == 0xC0) { /* 110xxxxx */
 		v = x & 0x1F;
 		i = 1;
 	}
-	else if(x & 0xE0 && !(x & 0x10)) { /* 1110xxxx */
+	else if((x & 0xF0) == 0xE0) { /* 1110xxxx */
 		v = x & 0x0F;
 		i = 2;
 	}
-	else if(x & 0xF0 && !(x & 0x08)) { /* 11110xxx */
+	else if((x & 0xF8) == 0xF0) { /* 11110xxx */
 		v = x & 0x07;
 		i = 3;
 	}
-	else if(x & 0xF8 && !(x & 0x04)) { /* 111110xx */
+	else if((x & 0xFC) == 0xF8) { /* 111110xx */
 		v = x & 0x03;
 		i = 4;
 	}
-	else if(x & 0xFC && !(x & 0x02)) { /* 1111110x */
+	else if((x & 0xFE) == 0xFC) { /* 1111110x */
 		v = x & 0x01;
 		i = 5;
 	}
-	else if(x & 0xFE && !(x & 0x01)) { /* 11111110 */
+	else if(x == 0xFE) { /* 11111110 */
 		v = 0;
 		i = 6;
 	}
