@@ -1,6 +1,6 @@
 /* grabbag - Convenience lib for various routines common to several tools
  * Copyright (C) 2002-2009  Josh Coalson
- * Copyright (C) 2011-2016  Xiph.Org Foundation
+ * Copyright (C) 2011-2025  Xiph.Org Foundation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 #include "FLAC/assert.h"
 #include "share/compat.h"
 #include "share/grabbag.h"
@@ -43,24 +44,6 @@ void grabbag__cuesheet_frame_to_msf(uint32_t frame, uint32_t *minutes, uint32_t
 	*minutes = frame;
 }
 
-/* since we only care about values >= 0 or error, returns < 0 for any illegal string, else value */
-static int local__parse_int_(const char *s)
-{
-	int ret = 0;
-	char c;
-
-	if(*s == '\0')
-		return -1;
-
-	while('\0' != (c = *s++))
-		if(c >= '0' && c <= '9')
-			ret = ret * 10 + (c - '0');
-		else
-			return -1;
-
-	return ret;
-}
-
 /* since we only care about values >= 0 or error, returns < 0 for any illegal string, else value */
 static FLAC__int64 local__parse_int64_(const char *s)
 {
@@ -71,14 +54,27 @@ static FLAC__int64 local__parse_int64_(const char *s)
 		return -1;
 
 	while('\0' != (c = *s++))
-		if(c >= '0' && c <= '9')
-			ret = ret * 10 + (c - '0');
+		if(c >= '0' && c <= '9') {
+			if(ret >= (INT64_MAX / 10))
+				return -1;
+			else
+				ret = ret * 10 + (c - '0');
+		}
 		else
 			return -1;
 
 	return ret;
 }
 
+/* since we only care about values >= 0 or error, returns < 0 for any illegal string, else value */
+static int local__parse_int_(const char *s)
+{
+	FLAC__int64 ret64 = local__parse_int64_(s);
+	if(ret64 < 0 || ret64 > INT_MAX)
+		return -1;
+	return ret64;
+}
+
 /* accept minute:second:frame syntax of '[0-9]+:[0-9][0-9]?:[0-9][0-9]?', but max second of 59 and max frame of 74, e.g. 0:0:0, 123:45:67
  * return sample number or <0 for error
  * WATCHOUT: if sample rate is not evenly divisible by 75, the resulting sample number will be approximate
@@ -88,18 +84,27 @@ static FLAC__int64 local__parse_msf_(const char *s, uint32_t sample_rate)
 	FLAC__int64 ret, field;
 	char c;
 
+	if(sample_rate == 0)
+		return -1;
+
 	c = *s++;
 	if(c >= '0' && c <= '9')
 		field = (c - '0');
 	else
 		return -1;
 	while(':' != (c = *s++)) {
-		if(c >= '0' && c <= '9')
-			field = field * 10 + (c - '0');
+		if(c >= '0' && c <= '9') {
+			if(field >= (INT64_MAX / 10))
+				return -1;
+			else
+				field = field * 10 + (c - '0');
+		}
 		else
 			return -1;
 	}
 
+	if(field >= INT64_MAX / (60 * sample_rate))
+		return -1;
 	ret = field * 60 * sample_rate;
 
 	c = *s++;
@@ -121,7 +126,12 @@ static FLAC__int64 local__parse_msf_(const char *s, uint32_t sample_rate)
 	if(field >= 60)
 		return -1;
 
-	ret += field * sample_rate;
+	{
+		FLAC__int64 tmp = ret;
+		ret += field * sample_rate;
+		if(ret < tmp)
+			return -1;
+	}
 
 	c = *s++;
 	if(c >= '0' && c <= '9')
@@ -143,7 +153,12 @@ static FLAC__int64 local__parse_msf_(const char *s, uint32_t sample_rate)
 	if(field >= 75)
 		return -1;
 
-	ret += field * (sample_rate / 75);
+	{
+		FLAC__int64 tmp = ret;
+		ret += field * (sample_rate / 75);
+		if(ret < tmp)
+			return -1;
+	}
 
 	return ret;
 }
@@ -158,21 +173,29 @@ static FLAC__int64 local__parse_ms_(const char *s, uint32_t sample_rate)
 	double x;
 	char c, *end;
 
+	if(sample_rate == 0)
+		return -1;
+
 	c = *s++;
 	if(c >= '0' && c <= '9')
 		field = (c - '0');
 	else
 		return -1;
 	while(':' != (c = *s++)) {
-		if(c >= '0' && c <= '9')
-			field = field * 10 + (c - '0');
+		if(c >= '0' && c <= '9') {
+			if(field >= (INT64_MAX / 10))
+				return -1;
+			else
+				field = field * 10 + (c - '0');
+		}
 		else
 			return -1;
 	}
 
+	if(field >= INT64_MAX / (60 * sample_rate))
+		return -1;
 	ret = field * 60 * sample_rate;
 
-	s++; /* skip the ':' */
 	if(strspn(s, "0123456789.") != strlen(s))
 		return -1;
 	x = strtod(s, &end);
@@ -309,7 +332,7 @@ static FLAC__bool local__cuesheet_parse_(FILE *file, const char **error_message,
 			}
 			else if(0 == FLAC__STRCASECMP(field, "INDEX")) {
 				FLAC__int64 xx;
-				FLAC__StreamMetadata_CueSheet_Track *track = &cs->tracks[cs->num_tracks-1];
+				FLAC__StreamMetadata_CueSheet_Track *track;
 				if(in_track_num < 0) {
 					*error_message = "found INDEX before any TRACK";
 					return false;
@@ -324,6 +347,7 @@ static FLAC__bool local__cuesheet_parse_(FILE *file, const char **error_message,
 					return false;
 				}
 				FLAC__ASSERT(cs->num_tracks > 0);
+				track = &cs->tracks[cs->num_tracks-1];
 				if(track->num_indices == 0) {
 					/* it's the first index point of the track */
 					if(in_index_num > 1) {
@@ -626,6 +650,8 @@ void grabbag__cuesheet_emit(FILE *file, const FLAC__StreamMetadata *cuesheet, co
 		fprintf(file, "CATALOG %s\n", cs->media_catalog_number);
 	fprintf(file, "FILE %s\n", file_reference);
 
+	FLAC__ASSERT(cs->num_tracks > 0);
+
 	for(track_num = 0; track_num < cs->num_tracks-1; track_num++) {
 		const FLAC__StreamMetadata_CueSheet_Track *track = cs->tracks + track_num;
 
