@@ -1,6 +1,6 @@
 /* test_libFLAC++ - Unit tester for libFLAC++
  * Copyright (C) 2002-2009  Josh Coalson
- * Copyright (C) 2011-2016  Xiph.Org Foundation
+ * Copyright (C) 2011-2025  Xiph.Org Foundation
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -60,8 +60,9 @@ static ::FLAC__StreamMetadata *expected_metadata_sequence_[9];
 static uint32_t num_expected_;
 static FLAC__off_t flacfilesize_;
 
-static const char *flacfilename(bool is_ogg)
+static const char *flacfilename(bool is_ogg, bool is_chained_ogg)
 {
+	if(is_chained_ogg) return "metadata_chained.oga";
 	return is_ogg? "metadata.oga" : "metadata.flac";
 }
 
@@ -95,9 +96,19 @@ static void free_metadata_blocks_()
 	mutils__free_metadata_blocks(&streaminfo_, &padding_, &seektable_, &application1_, &application2_, &vorbiscomment_, &cuesheet_, &picture_, &unknown_);
 }
 
-static bool generate_file_(FLAC__bool is_ogg)
+static FLAC__bool generate_file_(FLAC__bool is_ogg, FLAC__bool is_chained_ogg)
 {
-	printf("\n\ngenerating %sFLAC file for decoder tests...\n", is_ogg? "Ogg ":"");
+	FLAC__off_t filesize;
+	printf("\n\ngenerating %sFLAC file for decoder tests...\n", is_chained_ogg? "chained Ogg " : is_ogg? "Ogg " : "");
+
+	if(is_chained_ogg) {
+		/* Create a different file as the first link */
+		expected_metadata_sequence_[0] = &picture_;
+		if(!file_utils__generate_flacfile(is_ogg, flacfilename(is_ogg,true), &flacfilesize_, 512 * 1024, &streaminfo_, expected_metadata_sequence_, 1))
+			return die_("creating the encoded file");
+		file_utils__ogg_serial_number++;
+		filesize = flacfilesize_;
+	}
 
 	num_expected_ = 0;
 	expected_metadata_sequence_[num_expected_++] = &padding_;
@@ -110,9 +121,14 @@ static bool generate_file_(FLAC__bool is_ogg)
 	expected_metadata_sequence_[num_expected_++] = &unknown_;
 	/* WATCHOUT: for Ogg FLAC the encoder should move the VORBIS_COMMENT block to the front, right after STREAMINFO */
 
-	if(!file_utils__generate_flacfile(is_ogg, flacfilename(is_ogg), &flacfilesize_, 512 * 1024, &streaminfo_, expected_metadata_sequence_, num_expected_))
+	if(!file_utils__generate_flacfile(is_ogg, flacfilename(is_ogg,false), &flacfilesize_, 512 * 1024, &streaminfo_, expected_metadata_sequence_, num_expected_))
 		return die_("creating the encoded file");
 
+	if(is_chained_ogg) {
+		file_utils__append_file(flacfilename(is_ogg, true), flacfilename(is_ogg, false));
+		flacfilesize_ += filesize;
+	}
+
 	return true;
 }
 
@@ -121,10 +137,12 @@ class DecoderCommon {
 public:
 	Layer layer_;
 	uint32_t current_metadata_number_;
+	bool got_audio_;
 	bool ignore_errors_;
 	bool error_occurred_;
+	bool mute_test_;
 
-	DecoderCommon(Layer layer): layer_(layer), current_metadata_number_(0), ignore_errors_(false), error_occurred_(false) { }
+	DecoderCommon(Layer layer): layer_(layer), current_metadata_number_(0), got_audio_(false), ignore_errors_(false), error_occurred_(false), mute_test_(false) { }
 	virtual ~DecoderCommon(void) { }
 	::FLAC__StreamDecoderWriteStatus common_write_callback_(const ::FLAC__Frame *frame);
 	void common_metadata_callback_(const ::FLAC__StreamMetadata *metadata);
@@ -136,6 +154,7 @@ public:
 	if(error_occurred_)
 		return ::FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
 
+	got_audio_ = true;
 	if(
 		(frame->header.number_type == ::FLAC__FRAME_NUMBER_TYPE_FRAME_NUMBER && frame->header.number.frame_number == 0) ||
 		(frame->header.number_type == ::FLAC__FRAME_NUMBER_TYPE_SAMPLE_NUMBER && frame->header.number.sample_number == 0)
@@ -155,17 +174,19 @@ void DecoderCommon::common_metadata_callback_(const ::FLAC__StreamMetadata *meta
 	printf("%u... ", current_metadata_number_);
 	fflush(stdout);
 
-	if(current_metadata_number_ >= num_expected_) {
-		(void)die_("got more metadata blocks than expected");
-		error_occurred_ = true;
-	}
-	else {
-		if(!::FLAC__metadata_object_is_equal(expected_metadata_sequence_[current_metadata_number_], metadata)) {
-			(void)die_("metadata block mismatch");
+	if(!mute_test_) {
+		if(current_metadata_number_ >= num_expected_) {
+			(void)die_("got more metadata blocks than expected");
 			error_occurred_ = true;
 		}
+		else {
+			if(!::FLAC__metadata_object_is_equal(expected_metadata_sequence_[current_metadata_number_], metadata)) {
+				(void)die_("metadata block mismatch");
+				error_occurred_ = true;
+			}
+		}
+		current_metadata_number_++;
 	}
-	current_metadata_number_++;
 }
 
 void DecoderCommon::common_error_callback_(::FLAC__StreamDecoderErrorStatus status)
@@ -193,7 +214,7 @@ public:
 	void metadata_callback(const ::FLAC__StreamMetadata *metadata);
 	void error_callback(::FLAC__StreamDecoderErrorStatus status);
 
-	bool test_respond(bool is_ogg);
+	bool test_respond(bool is_ogg, bool is_chained_ogg);
 private:
 	StreamDecoder(const StreamDecoder&);
 	StreamDecoder&operator=(const StreamDecoder&);
@@ -301,7 +322,7 @@ void StreamDecoder::error_callback(::FLAC__StreamDecoderErrorStatus status)
 	common_error_callback_(status);
 }
 
-bool StreamDecoder::test_respond(bool is_ogg)
+bool StreamDecoder::test_respond(bool is_ogg, bool is_chained_ogg)
 {
 	::FLAC__StreamDecoderInitStatus init_status;
 
@@ -310,6 +331,13 @@ bool StreamDecoder::test_respond(bool is_ogg)
 		return false;
 	}
 
+	if(is_chained_ogg) {
+		printf("testing set_decode_chained_stream()... ");
+		if(!set_decode_chained_stream(true))
+			return die_s_("returned false", this);
+		printf("OK\n");
+	}
+
 	printf("testing init%s()... ", is_ogg? "_ogg":"");
 	init_status = is_ogg? init_ogg() : init();
 	if(init_status != ::FLAC__STREAM_DECODER_INIT_STATUS_OK)
@@ -323,6 +351,13 @@ bool StreamDecoder::test_respond(bool is_ogg)
 		return false;
 	}
 
+	if(is_chained_ogg) {
+		printf("skip first chain link with skip_single_link()... ");
+		if(!skip_single_link())
+			return die_s_("returned false", this);
+		printf("OK\n");
+	}
+
 	printf("testing process_until_end_of_stream()... ");
 	if(!process_until_end_of_stream()) {
 		State state = get_state();
@@ -352,7 +387,7 @@ public:
 	void metadata_callback(const ::FLAC__StreamMetadata *metadata);
 	void error_callback(::FLAC__StreamDecoderErrorStatus status);
 
-	bool test_respond(bool is_ogg);
+	bool test_respond(bool is_ogg, bool is_chained_ogg);
 };
 
 ::FLAC__StreamDecoderWriteStatus FileDecoder::write_callback(const ::FLAC__Frame *frame, const FLAC__int32 * const buffer[])
@@ -371,7 +406,7 @@ void FileDecoder::error_callback(::FLAC__StreamDecoderErrorStatus status)
 	common_error_callback_(status);
 }
 
-bool FileDecoder::test_respond(bool is_ogg)
+bool FileDecoder::test_respond(bool is_ogg, bool is_chained_ogg)
 {
 	::FLAC__StreamDecoderInitStatus init_status;
 
@@ -380,11 +415,18 @@ bool FileDecoder::test_respond(bool is_ogg)
 		return false;
 	}
 
+	if(is_chained_ogg) {
+		printf("testing set_decode_chained_stream()... ");
+		if(!set_decode_chained_stream(true))
+			return die_s_("returned false", this);
+		printf("OK\n");
+	}
+
 	switch(layer_) {
 		case LAYER_FILE:
 			{
 				printf("opening %sFLAC file... ", is_ogg? "Ogg ":"");
-				FILE *file = ::flac_fopen(flacfilename(is_ogg), "rb");
+				FILE *file = ::flac_fopen(flacfilename(is_ogg, is_chained_ogg), "rb");
 				if(0 == file) {
 					printf("ERROR (%s)\n", strerror(errno));
 					return false;
@@ -397,7 +439,7 @@ bool FileDecoder::test_respond(bool is_ogg)
 			break;
 		case LAYER_FILENAME:
 			printf("testing init%s()... ", is_ogg? "_ogg":"");
-			init_status = is_ogg? init_ogg(flacfilename(is_ogg)) : init(flacfilename(is_ogg));
+			init_status = is_ogg? init_ogg(flacfilename(is_ogg, is_chained_ogg)) : init(flacfilename(is_ogg, is_chained_ogg));
 			break;
 		default:
 			die_("internal error 001");
@@ -409,6 +451,20 @@ bool FileDecoder::test_respond(bool is_ogg)
 
 	current_metadata_number_ = 0;
 
+	if(is_chained_ogg) {
+		mute_test_ = true;
+		printf("process first chain link with process_until_end_of_link()... ");
+		if(!process_until_end_of_link())
+			return die_s_("returned false", this);
+		printf("OK\n");
+		mute_test_ = false;
+
+		printf("progress to next chain link with finish_link()... ");
+		if(!finish_link())
+			return die_s_("returned false", this);
+		printf("OK\n");
+	}
+
 	printf("testing process_until_end_of_stream()... ");
 	if(!process_until_end_of_stream()) {
 		State state = get_state();
@@ -437,13 +493,13 @@ static FLAC::Decoder::Stream *new_by_layer(Layer layer)
 		return new FileDecoder(layer);
 }
 
-static bool test_stream_decoder(Layer layer, bool is_ogg)
+static bool test_stream_decoder(Layer layer, bool is_ogg, bool is_chained_ogg)
 {
 	FLAC::Decoder::Stream *decoder;
 	::FLAC__StreamDecoderInitStatus init_status;
 	bool expect;
 
-	printf("\n+++ libFLAC++ unit test: FLAC::Decoder::%s (layer: %s, format: %s)\n\n", layer<LAYER_FILE? "Stream":"File", LayerString[layer], is_ogg? "Ogg FLAC" : "FLAC");
+	printf("\n+++ libFLAC++ unit test: FLAC::Decoder::%s (layer: %s, format: %s)\n\n", layer<LAYER_FILE? "Stream":"File", LayerString[layer], is_chained_ogg? "chained Ogg FLAC" : is_ogg? "Ogg FLAC" : "FLAC");
 
 	//
 	// test new -> delete
@@ -501,8 +557,8 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 			break;
 		case LAYER_FILENAME:
 			init_status = is_ogg?
-				dynamic_cast<FLAC::Decoder::File*>(decoder)->init_ogg(flacfilename(is_ogg)) :
-				dynamic_cast<FLAC::Decoder::File*>(decoder)->init(flacfilename(is_ogg));
+				dynamic_cast<FLAC::Decoder::File*>(decoder)->init_ogg(flacfilename(is_ogg, is_chained_ogg)) :
+				dynamic_cast<FLAC::Decoder::File*>(decoder)->init(flacfilename(is_ogg, is_chained_ogg));
 			break;
 		default:
 			die_("internal error 006");
@@ -551,11 +607,18 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 		return false;
 	}
 
+	if(is_chained_ogg) {
+		printf("testing set_decode_chained_stream()... ");
+		if(!decoder->set_decode_chained_stream(true))
+			return die_s_("returned false", decoder);
+		printf("OK\n");
+	}
+
 	switch(layer) {
 		case LAYER_STREAM:
 		case LAYER_SEEKABLE_STREAM:
 			printf("opening %sFLAC file... ", is_ogg? "Ogg ":"");
-			dynamic_cast<StreamDecoder*>(decoder)->file_ = ::flac_fopen(flacfilename(is_ogg), "rb");
+			dynamic_cast<StreamDecoder*>(decoder)->file_ = ::flac_fopen(flacfilename(is_ogg, is_chained_ogg), "rb");
 			if(0 == dynamic_cast<StreamDecoder*>(decoder)->file_) {
 				printf("ERROR (%s)\n", strerror(errno));
 				return false;
@@ -568,7 +631,7 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 		case LAYER_FILE:
 			{
 				printf("opening FLAC file... ");
-				FILE *file = ::flac_fopen(flacfilename(is_ogg), "rb");
+				FILE *file = ::flac_fopen(flacfilename(is_ogg, is_chained_ogg), "rb");
 				if(0 == file) {
 					printf("ERROR (%s)\n", strerror(errno));
 					return false;
@@ -584,8 +647,8 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 		case LAYER_FILENAME:
 			printf("testing init%s()... ", is_ogg? "_ogg":"");
 			init_status = is_ogg?
-				dynamic_cast<FLAC::Decoder::File*>(decoder)->init_ogg(flacfilename(is_ogg)) :
-				dynamic_cast<FLAC::Decoder::File*>(decoder)->init(flacfilename(is_ogg));
+				dynamic_cast<FLAC::Decoder::File*>(decoder)->init_ogg(flacfilename(is_ogg, is_chained_ogg)) :
+				dynamic_cast<FLAC::Decoder::File*>(decoder)->init(flacfilename(is_ogg, is_chained_ogg));
 			break;
 		default:
 			die_("internal error 009");
@@ -595,14 +658,46 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 		return die_s_(0, decoder);
 	printf("OK\n");
 
-	printf("testing get_state()... ");
-	FLAC::Decoder::Stream::State state = decoder->get_state();
-	printf("returned state = %u (%s)... OK\n", (uint32_t)((::FLAC__StreamDecoderState)state), state.as_cstring());
-
 	dynamic_cast<DecoderCommon*>(decoder)->current_metadata_number_ = 0;
 	dynamic_cast<DecoderCommon*>(decoder)->ignore_errors_ = false;
 	dynamic_cast<DecoderCommon*>(decoder)->error_occurred_ = false;
 
+	if(is_chained_ogg) {
+		printf("process first chain link with process_until_end_of_link()... ");
+		if(!decoder->process_until_end_of_link())
+			return die_s_("returned false", decoder);
+		printf("OK\n");
+
+                printf("checking whether metadata was returned by previous call... ");
+                if(dynamic_cast<DecoderCommon*>(decoder)->current_metadata_number_ != 1) {
+                        return die_s_("no metadata blocks returned", decoder);
+                }
+                printf("OK\n");
+		dynamic_cast<DecoderCommon*>(decoder)->current_metadata_number_ = 0;
+
+		printf("testing get_state()... ");
+		FLAC::Decoder::Stream::State state = decoder->get_state();
+		printf("returned state = %u (%s)... OK\n", (uint32_t)((::FLAC__StreamDecoderState)state), state.as_cstring());
+
+		printf("testing get_link_lengths()... ");
+		if(decoder->get_link_lengths(NULL) != FLAC__STREAM_DECODER_GET_LINK_LENGTHS_NOT_INDEXED)
+			return die_s_("returned incorrectly", decoder);
+
+		printf("progress to next chain link with finish_link()... ");
+		if(!decoder->finish_link())
+			return die_s_("returned false", decoder);
+		printf("OK\n");
+	}
+	else {
+		printf("testing get_link_lengths()... ");
+		if(decoder->get_link_lengths(NULL) != FLAC__STREAM_DECODER_GET_LINK_LENGTHS_INVALID)
+			return die_s_("returned incorrectly", decoder);
+	}
+
+	printf("testing get_state()... ");
+	FLAC::Decoder::Stream::State state = decoder->get_state();
+	printf("returned state = %u (%s)... OK\n", (uint32_t)((::FLAC__StreamDecoderState)state), state.as_cstring());
+
 	printf("testing get_md5_checking()... ");
 	if(!decoder->get_md5_checking()) {
 		printf("FAILED, returned false, expected true\n");
@@ -639,12 +734,42 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 		dynamic_cast<DecoderCommon*>(decoder)->ignore_errors_ = false;
 	}
 
+	if(is_chained_ogg) {
+		dynamic_cast<DecoderCommon*>(decoder)->current_metadata_number_ = 0;
+	}
+
 	expect = (layer != LAYER_STREAM);
 	printf("testing seek_absolute()... ");
 	if(decoder->seek_absolute(0) != expect)
 		return die_s_(expect? "returned false" : "returned true", decoder);
 	printf("OK\n");
 
+        if(is_chained_ogg) {
+		if(layer != LAYER_STREAM) {
+			printf("checking whether metadata was returned by previous call... ");
+				if(dynamic_cast<DecoderCommon*>(decoder)->current_metadata_number_ != 1) {
+				return die_s_("no metadata blocks returned", decoder);
+			}
+			printf("OK\n");
+			dynamic_cast<DecoderCommon*>(decoder)->current_metadata_number_ = 0;
+		}
+
+                printf("testing seek_absolute(), seeking to second link... ");
+                if(decoder->seek_absolute(512 * 1024) != expect)
+                        return die_s_(expect? "returned false" : "returned true", decoder);
+                printf("OK\n");
+
+		if(layer != LAYER_STREAM) {
+			printf("checking whether metadata was returned by previous call... ");
+				if(dynamic_cast<DecoderCommon*>(decoder)->current_metadata_number_ != 1) {
+				return die_s_("no metadata blocks returned", decoder);
+			}
+			printf("OK\n");
+			dynamic_cast<DecoderCommon*>(decoder)->current_metadata_number_ = 0;
+		}
+        }
+
+
 	printf("testing process_until_end_of_stream()... ");
 	if(!decoder->process_until_end_of_stream())
 		return die_s_("returned false", decoder);
@@ -656,6 +781,24 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 		return die_s_(expect? "returned false" : "returned true", decoder);
 	printf("OK\n");
 
+	if(is_chained_ogg) {
+		FLAC__uint64 *link_lengths;
+		printf("testing get_link_lengths()... ");
+		if(decoder->get_link_lengths(NULL) != 2)
+			return die_s_("returned incorrectly", decoder);
+
+		printf("testing get_link_lengths()... ");
+		if(decoder->get_link_lengths(&link_lengths) != 2)
+			return die_s_("returned incorrectly", decoder);
+		free(link_lengths);
+	}
+	else {
+		printf("testing get_link_lengths()... ");
+		if(decoder->get_link_lengths(NULL) != FLAC__STREAM_DECODER_GET_LINK_LENGTHS_INVALID)
+			return die_s_("returned incorrectly", decoder);
+	}
+
+
 	printf("testing get_channels()... ");
 	{
 		uint32_t channels = decoder->get_channels();
@@ -719,6 +862,20 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 
 		dynamic_cast<DecoderCommon*>(decoder)->current_metadata_number_ = 0;
 
+		if(is_chained_ogg) {
+			dynamic_cast<DecoderCommon*>(decoder)->mute_test_ = true;
+			printf("process first chain link with process_until_end_of_link()... ");
+			if(!decoder->process_until_end_of_link())
+				return die_s_("returned false", decoder);
+			printf("OK\n");
+			dynamic_cast<DecoderCommon*>(decoder)->mute_test_ = false;
+
+			printf("progress to next chain link with finish_link()... ");
+			if(!decoder->finish_link())
+				return die_s_("returned false", decoder);
+			printf("OK\n");
+		}
+
 		printf("testing process_until_end_of_stream()... ");
 		if(!decoder->process_until_end_of_stream())
 			return die_s_("returned false", decoder);
@@ -745,11 +902,10 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 	printf("OK\n");
 
 	num_expected_ = 0;
-	if(is_ogg) { /* encoder moves vorbis comment after streaminfo according to ogg mapping */
+	if(is_ogg) { /* encoder moves vorbis comment after streaminfo according to ogg mapping. Also removes the seektable */
 		expected_metadata_sequence_[num_expected_++] = &streaminfo_;
 		expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
 		expected_metadata_sequence_[num_expected_++] = &padding_;
-		expected_metadata_sequence_[num_expected_++] = &seektable_;
 		expected_metadata_sequence_[num_expected_++] = &application1_;
 		expected_metadata_sequence_[num_expected_++] = &application2_;
 		expected_metadata_sequence_[num_expected_++] = &cuesheet_;
@@ -768,7 +924,7 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 		expected_metadata_sequence_[num_expected_++] = &unknown_;
 	}
 
-	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg)))
+	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg)))
 		return false;
 
 	/*
@@ -784,7 +940,7 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 
 	num_expected_ = 0;
 
-	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg)))
+	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg)))
 		return false;
 
 	/*
@@ -808,14 +964,15 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 	num_expected_ = 0;
 	expected_metadata_sequence_[num_expected_++] = &streaminfo_;
 	expected_metadata_sequence_[num_expected_++] = &padding_;
-	expected_metadata_sequence_[num_expected_++] = &seektable_;
+	if(!is_ogg) /* encoder removes seektable for ogg */
+		expected_metadata_sequence_[num_expected_++] = &seektable_;
 	expected_metadata_sequence_[num_expected_++] = &application1_;
 	expected_metadata_sequence_[num_expected_++] = &application2_;
 	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
 	expected_metadata_sequence_[num_expected_++] = &picture_;
 	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
-	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg)))
+	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg)))
 		return false;
 
 	/*
@@ -837,11 +994,10 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 	printf("OK\n");
 
 	num_expected_ = 0;
-	if(is_ogg) { /* encoder moves vorbis comment after streaminfo according to ogg mapping */
+	if(is_ogg) { /* encoder moves vorbis comment after streaminfo according to ogg mapping. Also removes the seektable */
 		expected_metadata_sequence_[num_expected_++] = &streaminfo_;
 		expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
 		expected_metadata_sequence_[num_expected_++] = &padding_;
-		expected_metadata_sequence_[num_expected_++] = &seektable_;
 		expected_metadata_sequence_[num_expected_++] = &cuesheet_;
 		expected_metadata_sequence_[num_expected_++] = &picture_;
 		expected_metadata_sequence_[num_expected_++] = &unknown_;
@@ -856,7 +1012,7 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 		expected_metadata_sequence_[num_expected_++] = &unknown_;
 	}
 
-	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg)))
+	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg)))
 		return false;
 
 	/*
@@ -878,11 +1034,10 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 	printf("OK\n");
 
 	num_expected_ = 0;
-	if(is_ogg) { /* encoder moves vorbis comment after streaminfo according to ogg mapping */
+	if(is_ogg) { /* encoder moves vorbis comment after streaminfo according to ogg mapping. Also removes the seektable */
 		expected_metadata_sequence_[num_expected_++] = &streaminfo_;
 		expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
 		expected_metadata_sequence_[num_expected_++] = &padding_;
-		expected_metadata_sequence_[num_expected_++] = &seektable_;
 		expected_metadata_sequence_[num_expected_++] = &application2_;
 		expected_metadata_sequence_[num_expected_++] = &cuesheet_;
 		expected_metadata_sequence_[num_expected_++] = &picture_;
@@ -899,7 +1054,7 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 		expected_metadata_sequence_[num_expected_++] = &unknown_;
 	}
 
-	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg)))
+	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg)))
 		return false;
 
 	/*
@@ -928,11 +1083,10 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 	printf("OK\n");
 
 	num_expected_ = 0;
-	if(is_ogg) { /* encoder moves vorbis comment after streaminfo according to ogg mapping */
+	if(is_ogg) { /* encoder moves vorbis comment after streaminfo according to ogg mapping. Also removes seektable */
 		expected_metadata_sequence_[num_expected_++] = &streaminfo_;
 		expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
 		expected_metadata_sequence_[num_expected_++] = &padding_;
-		expected_metadata_sequence_[num_expected_++] = &seektable_;
 		expected_metadata_sequence_[num_expected_++] = &cuesheet_;
 		expected_metadata_sequence_[num_expected_++] = &picture_;
 		expected_metadata_sequence_[num_expected_++] = &unknown_;
@@ -947,7 +1101,7 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 		expected_metadata_sequence_[num_expected_++] = &unknown_;
 	}
 
-	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg)))
+	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg)))
 		return false;
 
 	/*
@@ -971,7 +1125,7 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 	num_expected_ = 0;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
 
-	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg)))
+	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg)))
 		return false;
 
 	/*
@@ -996,7 +1150,7 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 	expected_metadata_sequence_[num_expected_++] = &application1_;
 	expected_metadata_sequence_[num_expected_++] = &application2_;
 
-	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg)))
+	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg)))
 		return false;
 
 	/*
@@ -1020,7 +1174,7 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 	num_expected_ = 0;
 	expected_metadata_sequence_[num_expected_++] = &application1_;
 
-	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg)))
+	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg)))
 		return false;
 
 	/*
@@ -1052,7 +1206,7 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 	expected_metadata_sequence_[num_expected_++] = &application1_;
 	expected_metadata_sequence_[num_expected_++] = &application2_;
 
-	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg)))
+	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg)))
 		return false;
 
 	/*
@@ -1081,11 +1235,10 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 	printf("OK\n");
 
 	num_expected_ = 0;
-	if(is_ogg) { /* encoder moves vorbis comment after streaminfo according to ogg mapping */
+	if(is_ogg) { /* encoder moves vorbis comment after streaminfo according to ogg mapping. Also removes the seektable */
 		expected_metadata_sequence_[num_expected_++] = &streaminfo_;
 		expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
 		expected_metadata_sequence_[num_expected_++] = &padding_;
-		expected_metadata_sequence_[num_expected_++] = &seektable_;
 		expected_metadata_sequence_[num_expected_++] = &application1_;
 		expected_metadata_sequence_[num_expected_++] = &cuesheet_;
 		expected_metadata_sequence_[num_expected_++] = &picture_;
@@ -1102,7 +1255,7 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 		expected_metadata_sequence_[num_expected_++] = &unknown_;
 	}
 
-	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg)))
+	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg)))
 		return false;
 
 	/*
@@ -1133,7 +1286,7 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 	num_expected_ = 0;
 	expected_metadata_sequence_[num_expected_++] = &application2_;
 
-	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg)))
+	if(!(layer < LAYER_FILE? dynamic_cast<StreamDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg) : dynamic_cast<FileDecoder*>(decoder)->test_respond(is_ogg, is_chained_ogg)))
 		return false;
 
 	if(layer < LAYER_FILE) /* for LAYER_FILE, FLAC__stream_decoder_finish() closes the file */
@@ -1151,31 +1304,34 @@ static bool test_stream_decoder(Layer layer, bool is_ogg)
 bool test_decoders()
 {
 	FLAC__bool is_ogg = false;
+	FLAC__bool is_chained_ogg = false;
 
 	while(1) {
 		init_metadata_blocks_();
 
-		if(!generate_file_(is_ogg))
+		if(!generate_file_(is_ogg, is_chained_ogg))
 			return false;
 
-		if(!test_stream_decoder(LAYER_STREAM, is_ogg))
+		if(!test_stream_decoder(LAYER_STREAM, is_ogg, is_chained_ogg))
 			return false;
 
-		if(!test_stream_decoder(LAYER_SEEKABLE_STREAM, is_ogg))
+		if(!test_stream_decoder(LAYER_SEEKABLE_STREAM, is_ogg, is_chained_ogg))
 			return false;
 
-		if(!test_stream_decoder(LAYER_FILE, is_ogg))
+		if(!test_stream_decoder(LAYER_FILE, is_ogg, is_chained_ogg))
 			return false;
 
-		if(!test_stream_decoder(LAYER_FILENAME, is_ogg))
+		if(!test_stream_decoder(LAYER_FILENAME, is_ogg, is_chained_ogg))
 			return false;
 
-		(void) grabbag__file_remove_file(flacfilename(is_ogg));
+		(void) grabbag__file_remove_file(flacfilename(is_ogg, is_chained_ogg));
 
 		free_metadata_blocks_();
 
-		if(!FLAC_API_SUPPORTS_OGG_FLAC || is_ogg)
+		if(!FLAC_API_SUPPORTS_OGG_FLAC || is_chained_ogg)
 			break;
+		if(is_ogg)
+			is_chained_ogg = true;
 		is_ogg = true;
 	}
 
